@@ -1,5 +1,5 @@
 /*  malloc.c - Dumb memory allocation routines
- *  
+ *
  *  Copyright (C) 1997 Paul Mackerras
  *                1996 Maurizio Plaza
  *                1996 Jakub Jelinek
@@ -21,6 +21,7 @@
 
 #include "types.h"
 #include "stddef.h"
+#include "string.h"
 
 /* Imported functions */
 extern void prom_printf (char *fmt, ...);
